/******************************************************************************
 * Server.java - The primary server thread.
 * $Id: Server.java,v 1.10 2011/06/11 19:18:35 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Server.java,v $
 * Revision 1.10  2011/06/11 19:18:35  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.9  2011/06/09 18:28:34  dick
 * Whitespace.
 *
 * Revision 1.8  2011/06/08 20:58:09  dick
 * Leave_Command.execute() gets a shuttingDown parameter from us to prevent sending bogus leave messages. (Still need to send server shutting down messages).
 *
 * Revision 1.7  2011/06/06 16:17:20  dick
 * Substitute the version number when streaming login.txt
 *
 * Revision 1.6  2011/06/06 03:18:26  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.5  2011/06/06 02:39:27  dick
 * Move the Player functions to JibsServer, which is more appropriate.
 * This guy really is low level thread handling, not "isPlayerOnline()".
 *
 * Revision 1.4  2011/05/31 17:48:50  dick
 * Fix all warnings from the import.
 *
 * Revision 1.3  2011/05/31 15:25:37  dick
 * Fix cvs link.
 *
 * Revision 1.2  2011/05/30 04:24:45  dick
 * Read the logo (login message) from the resources.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import net.sourceforge.jibs.command.Exit_Command;
import net.sourceforge.jibs.command.Leave_Command;
import net.sourceforge.jibs.util.JibsShutdown;
import net.sourceforge.jibs.util.JibsWriter;

/** The primary server thread.
 * Contains the socket listener, which spawns login threads.
 * Contains the master lists
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.10 $ <br> $Date: 2011/06/11 19:18:35 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/server/Server.java">cvs Server.java</a>
 */
public class Server implements Runnable {
//	private static final Logger logger = Logger.getLogger(Server.class);
	private ServerSocket listener;
	private JibsServer jibsServer;
	private boolean runs;

	public Server(ServerSocket listener, int portno) {
		this.listener = listener;
		runs = true;
		jibsServer = JibsServer.getInstance();
	}

	/** The primary jIBS server thread.  
	 * Listens on a socket and dispatches a new ClientWorker in a new thread.
	 */
	public void run() {
		int loginCount = 0;

		while (runs) {
			Socket client;
			BufferedReader in;
			JibsWriter out;
			boolean bLogin = true;

			try {
				client = listener.accept();

				in = new BufferedReader(new InputStreamReader(client.getInputStream()));
				out = new JibsWriter(client.getOutputStream());

				JibsShutdown jibsShutdown = jibsServer.getJibsShutdown();

				if (jibsShutdown != null) {
					Date dt = jibsShutdown.getShutdownDate();
					Date now = new Date();
					long diff = Math.abs(now.getTime() - dt.getTime());

					if (diff < (10 * 60 * 1000)) {
						out.println("jIBS will shutdown. No login allowed.");
						client.close();
						bLogin = false;
					}
				}

				if (bLogin) {
					ClientWorker w = new ClientWorker(client, in, out);
					Thread t = new Thread(w, "ClientWorker-" + loginCount++);
					t.start();
				}
			} catch (IOException e) {
				boolean doLog = e.getMessage()
				.equalsIgnoreCase("Socket is closed");
				doLog |= e.getMessage().equalsIgnoreCase("socket closed");

				if (!doLog) {
					jibsServer.logException(e);
				}
			}
		}
	}

	public ServerSocket getListener() {
		return listener;
	}

	public void setRuns(boolean runs) {
		this.runs = runs;
	}

	/** Send a logoff to, and disconnect all of the clients
	 */
	@SuppressWarnings("unchecked")
	public void closeAllClients() {
		// DIK: Why is this a clone of the clients map?
		Map<String, Player> omap = (Map) jibsServer.getAllClients().clone();
		// DIK? And why is it synchronized? It seems pretty private to this function
		Map map = Collections.synchronizedMap(omap);

		synchronized (map) {
			Set set = map.entrySet();
			Iterator iter = set.iterator();
			Leave_Command leaveCommand = new Leave_Command();
			Exit_Command exitCommand = new Exit_Command();		// XXX: Need to send server shutting down messages.

			while (iter.hasNext()) {
				Entry entry = (Entry) iter.next();
				Player curPlayer = (Player) entry.getValue();
				leaveCommand.execute(jibsServer, curPlayer, true);
				exitCommand.displayLogoff(jibsServer,
						curPlayer);

				ClientWorker cw = curPlayer.getClientWorker();

				if (cw != null) {
					cw.disConnectPlayer(curPlayer);
					cw.stopWatchThread();

					try {
						cw.getSocket().close();
					} catch (IOException e) {
						jibsServer.logException(e);
					}
				}
			}
		}
	}
}
